@@ -65,6 +65,9 @@ extern SSL_CTX *ctx;				  /* xchat.c */
 static struct session *g_sess = NULL;
 #endif
 
+static GSList *away_list = NULL;
+GSList *serv_list = NULL;
+
 static void auto_reconnect (server *serv, int send_quit, int err);
 static void server_disconnect (session * sess, int sendquit, int err);
 static int server_cleanup (server * serv);
@@ -228,8 +231,9 @@ void
 tcp_sendf (server *serv, char *fmt, ...)
 {
 	va_list args;
-	/* keep this buffer in BSS */
-	static char send_buf[520];	/* good code hey (no it's not overflowable) */
+	/* keep this buffer in BSS. Converting UTF-8 to ISO-8859-x might make the
+      string shorter, so allow alot more than 512 for now. */
+	static char send_buf[1540];	/* good code hey (no it's not overflowable) */
 	int len;
 
 	va_start (args, fmt);
@@ -267,8 +271,8 @@ server_inline (server *serv, char *line, int len)
 	/* Checks whether we're set to use UTF-8 charset */
 	if ((serv->encoding == NULL && prefs.utf8_locale) ||
 	    (serv->encoding != NULL &&
-		 (strcasecmp (serv->encoding, "utf8") == 0 ||
-		  strcasecmp (serv->encoding, "utf-8") == 0)))
+		 (strcasecmp (serv->encoding, "UTF8") == 0 ||
+		  strcasecmp (serv->encoding, "UTF-8") == 0)))
 	{
 		/* The user has the UTF-8 charset set, either via /charset
 		command or from his UTF-8 locale. Thus, we first try the
@@ -318,7 +322,7 @@ server_inline (server *serv, char *line, int len)
 				utf_line_allocated = g_convert_with_fallback (conv_line, conv_len, "UTF-8", encoding, "?", &read_len, &utf_len, &err);
 				if (err != NULL)
 				{
-					if (err->code == G_CONVERT_ERROR_ILLEGAL_SEQUENCE)
+					if (err->code == G_CONVERT_ERROR_ILLEGAL_SEQUENCE && conv_len > (read_len + 1))
 					{
 						/* Make our best bet by removing the erroneous char.
 						   This will work for casual 8-bit strings with non-standard chars. */
@@ -350,7 +354,7 @@ server_inline (server *serv, char *line, int len)
 	}
 
 	fe_add_rawlog (serv, line, len, FALSE);
-	url_check (line);
+	url_check_line (line, len);
 
 	/* let proto-irc.c handle it */
 	serv->p_inline (serv, line, len);
@@ -384,7 +388,7 @@ server_read (GIOChannel *source, GIOCondition condition, server *serv)
 			error = 0;
 			if (len < 0)
 			{
-				if (would_block_again ())
+				if (would_block ())
 					return TRUE;
 				error = sock_error ();
 			}
@@ -459,10 +463,12 @@ server_connected (server * serv)
 		if (serv->network)
 		{
 			serv->p_login (serv,
-								(((ircnet *)serv->network)->user) ?
+								(!(((ircnet *)serv->network)->flags & FLAG_USE_GLOBAL) &&
+								 (((ircnet *)serv->network)->user)) ?
 								(((ircnet *)serv->network)->user) :
 								prefs.username,
-								(((ircnet *)serv->network)->real) ?
+								(!(((ircnet *)serv->network)->flags & FLAG_USE_GLOBAL) &&
+								 (((ircnet *)serv->network)->real)) ?
 								(((ircnet *)serv->network)->real) :
 								prefs.realname);
 		} else
@@ -475,7 +481,8 @@ server_connected (server * serv)
 						 NULL, NULL, 0);
 	}
 
-	set_server_name (serv, serv->servername);
+	server_set_name (serv, serv->servername);
+	fe_server_event (serv, FE_SE_CONNECT, 0);
 }
 
 #ifdef WIN32
@@ -500,6 +507,12 @@ server_stopconnecting (server * serv)
 		serv->iotag = 0;
 	}
 
+	if (serv->joindelay_tag)
+	{
+		fe_timeout_remove (serv->joindelay_tag);
+		serv->joindelay_tag = 0;
+	}
+
 #ifndef WIN32
 	/* kill the child process trying to connect */
 	kill (serv->childpid, SIGKILL);
@@ -779,6 +792,7 @@ auto_reconnect (server *serv, int send_quit, int err)
 		serv->reconnect_away = serv->is_away;
 
 	serv->recondelay_tag = fe_timeout_add (del, timeout_auto_reconnect, serv);
+	fe_server_event (serv, FE_SE_RECONDELAY, del);
 }
 
 static void
@@ -972,6 +986,12 @@ server_cleanup (server * serv)
 		serv->iotag = 0;
 	}
 
+	if (serv->joindelay_tag)
+	{
+		fe_timeout_remove (serv->joindelay_tag);
+		serv->joindelay_tag = 0;
+	}
+
 #ifdef USE_OPENSSL
 	if (serv->ssl)
 	{
@@ -1021,6 +1041,8 @@ server_disconnect (session * sess, int sendquit, int err)
 		server_sendquit (sess);
 	}
 
+	fe_server_event (serv, FE_SE_DISCONNECT, 0);
+
 	/* close all sockets & io tags */
 	switch (server_cleanup (serv))
 	{
@@ -1547,8 +1569,8 @@ server_connect (server *serv, char *hostname, int port, int no_login)
 	}
 #endif
 	serv->childpid = pid;
-	serv->iotag =
-		fe_input_add (serv->childread, FIA_READ|FIA_FD, server_read_child, serv);
+	serv->iotag = fe_input_add (serv->childread, FIA_READ, server_read_child,
+										 serv);
 }
 
 void
@@ -1616,6 +1638,12 @@ server_new (void)
 	return serv;
 }
 
+int
+is_server (server *serv)
+{
+	return g_slist_find (serv_list, serv) ? 1 : 0;
+}
+
 void
 server_set_defaults (server *serv)
 {
@@ -1655,3 +1683,130 @@ server_get_network (server *serv, gboolean fallback)
 
 	return NULL;
 }
+
+void
+server_set_name (server *serv, char *name)
+{
+	GSList *list = sess_list;
+	session *sess;
+
+	if (name[0] == 0)
+		name = serv->hostname;
+
+	/* strncpy parameters must NOT overlap */
+	if (name != serv->servername)
+	{
+		safe_strcpy (serv->servername, name, sizeof (serv->servername));
+	}
+
+	while (list)
+	{
+		sess = (session *) list->data;
+		if (sess->server == serv)
+			fe_set_title (sess);
+		list = list->next;
+	}
+
+	if (serv->server_session->type == SESS_SERVER)
+	{
+		if (serv->network)
+		{
+			safe_strcpy (serv->server_session->channel, ((ircnet *)serv->network)->name, CHANLEN);
+		} else
+		{
+			safe_strcpy (serv->server_session->channel, name, CHANLEN);
+		}
+		fe_set_channel (serv->server_session);
+	}
+}
+
+struct away_msg *
+server_away_find_message (server *serv, char *nick)
+{
+	struct away_msg *away;
+	GSList *list = away_list;
+	while (list)
+	{
+		away = (struct away_msg *) list->data;
+		if (away->server == serv && !serv->p_cmp (nick, away->nick))
+			return away;
+		list = list->next;
+	}
+	return NULL;
+}
+
+static void
+server_away_free_messages (server *serv)
+{
+	GSList *list, *next;
+	struct away_msg *away;
+
+	list = away_list;
+	while (list)
+	{
+		away = list->data;
+		next = list->next;
+		if (away->server == serv)
+		{
+			away_list = g_slist_remove (away_list, away);
+			if (away->message)
+				free (away->message);
+			free (away);
+			next = away_list;
+		}
+		list = next;
+	}
+}
+
+void
+server_away_save_message (server *serv, char *nick, char *msg)
+{
+	struct away_msg *away = server_away_find_message (serv, nick);
+
+	if (away)						  /* Change message for known user */
+	{
+		if (away->message)
+			free (away->message);
+		away->message = strdup (msg);
+	} else
+		/* Create brand new entry */
+	{
+		away = malloc (sizeof (struct away_msg));
+		if (away)
+		{
+			away->server = serv;
+			safe_strcpy (away->nick, nick, sizeof (away->nick));
+			away->message = strdup (msg);
+			away_list = g_slist_prepend (away_list, away);
+		}
+	}
+}
+
+void
+server_free (server *serv)
+{
+	serv->cleanup (serv);
+
+	serv_list = g_slist_remove (serv_list, serv);
+
+	dcc_notify_kill (serv);
+	serv->flush_queue (serv);
+	server_away_free_messages (serv);
+
+	free (serv->nick_modes);
+	free (serv->nick_prefixes);
+	free (serv->chanmodes);
+	free (serv->chantypes);
+	if (serv->bad_nick_prefixes)
+		free (serv->bad_nick_prefixes);
+	if (serv->last_away_reason)
+		free (serv->last_away_reason);
+	if (serv->encoding)
+		free (serv->encoding);
+
+	fe_server_callback (serv);
+
+	free (serv);
+
+	notify_cleanup ();
+}
